//Ŀ
//                                                                           
//  Program Name: ScrolBar.prg      Copyright: WardStar Consulting           
//  Date Created: 17/07/95           Language: Clipper 5.2e                  
//  Time Created: 18:41:10             Author: Robin Ward                    
//                                                                           
//   Description:   Adds scroll bars to any tbrowse operating on a database  
//                  file that is not indexed.                                
//                                                                           
//                  You can add your own code block to enable updating of    
//                  indexed or non database tbrowses.                        
//                                                                           
//                                                                           
//VerRevisedDescriptionĴ
//                                                                           
//   1.00  17/07/95    Created                                               
//                                                                           
//   1.01  30/07/95    Add PanUp and PanDown()                               
//                                                                           
//
//   

#include "ScrolBar.ch"


// some defaults
#define  SB_FILL_CHAR         chr(177)
#define  SB_FLAG              "Scroll Bar"
#define  SB_COLOR             "w/n"


// four object variables
#define  BARCOLOR             qself()[ len(qself())-3 ]
#define  BARSTATUS            qself()[ len(qself())-2 ]
#define  BARMOVEBLK           qself()[ len(qself())-1 ]
#define  BARFLAG              qself()[ len(qself())   ]


//==================================
// This function adds the new methods and variables to all tbrowses
// that currently exist, and that will be created later.  It needs
// to be called only once in a program.
//
function AddScrollBars
local obj     := tbrowsenew()                    // get a tbrowse object
local nHandle := obj:classh                      // the handle for this class
local aClass  := __classsel( nHandle)            // array of methods, vars

// search array of methods to see if it has already been modified
if ascan( aClass, "BARINIT") == 0

   // add three public variables
   __ClassAdd( nHandle, "BarColor",    "__SB_Color"  )
   __ClassAdd( nHandle, "_BarColor",   "__SB_Color"  )
   __ClassAdd( nHandle, "BarMoveBlk",  "__SB_Move"   )
   __ClassAdd( nHandle, "_BarMoveBlk", "__SB_Move"   )
   __ClassAdd( nHandle, "BarStatus",   "__SB_Status" )
   __ClassAdd( nHandle, "_BarStatus",  "__SB_Status" )

   // add two methods
   __ClassAdd( nHandle, "BarInit",     "__SB_Init"   )
   __ClassAdd( nHandle, "BarUpdate",   "__SB_Update" )

   // add two more methods
   __ClassAdd( nHandle, "PanUp",       "__SB_PanUp"  )
   __ClassAdd( nHandle, "PanDown",     "__SB_PanDown")
endif
return nil


//==================================
// Each tbrowse object has to initialized with the extra variables
// that are required for the scrolbar methods.  This function
// extends the length of the tbrowse objects array by four variables.
//
function __SB_Init()
local obj := qself()

if BARFLAG != SB_FLAG
   aadd( obj, SB_COLOR )
   aadd( obj, SB_STATUS_INVALID )
   aadd( obj, { |o| BarMoveDbf(o)} )
   aadd( obj, SB_FLAG )
   obj:FootSep := SB_FILL_CHAR
   if obj:nRight == maxcol()
      obj:nRight--
   endif
   obj:configure()
endif
return qself()


//==================================
// Updates the horizontal and vertical scrollbars.
// Call after the object has been stabilized.
//
function __SB_Update()
local obj      := qself():BarInit()
local nTop     := obj:nBottom - obj:RowCount
local nBot     := obj:nBottom
local nLeft    := obj:nLeft
local nRight   := obj:nRight
local nHorCol
local nVertRow
local cPtr

DispBegin()

// draw the vertical scrolbar
@ nTop, nRight+1, nBot-1, nRight+1 box SB_FILL_CHAR

// calculate the vertical pointer position using the objects codeblock
nVertRow := (eval( obj:BarMoveBlk, obj) * (nBot-nTop-1 )) + 0.5

// is there a valid vertical pointer?
if obj:BarStatus != SB_STATUS_INVALID

   // choose the type of vertical pointer
   if obj:BarStatus == SB_STATUS_FIRST
      cPtr := chr(25)
   elseif obj:BarStatus == SB_STATUS_LAST
      cPtr := chr(24)
   else
      cPtr := chr(18)
   endif

   // display the vertical pointer
   @ nTop+nVertRow, nRight+1 say cPtr color obj:BarColor
endif


// draw the horizontal scrolbar
@ nBot, nLeft, nBot, nRight box SB_FILL_CHAR

// calculate the horizontal pointer position
nHorCol := (((obj:ColPos-1)/(obj:colcount-1)) * (nRight-nLeft)) + 0.5 + nLeft

// is there a valid horizontal pointer?
if obj:colcount > 1

   // choose the type of horizontal pointer
   if obj:ColPos == 1
      cPtr := chr(26)
   elseif obj:ColPos == obj:colcount
      cPtr := chr(27)
   else
      cPtr := chr(29)
   endif

   // display the horizontal pointer
   @ nBot, nHorCol say cPtr color obj:BarColor
endif

DispEnd()
return qself()


//==================================
function __SB_Color( cColor)

qself():BarInit()
if valtype(cColor) == "C"
   BARCOLOR := cColor
endif
return BARCOLOR


//==================================
function __SB_Status( nStatus)

qself():BarInit()
if valtype(nStatus) == "N"
   BARSTATUS := nStatus
endif
return BARSTATUS


//==================================
function __SB_Move( bPos)

qself():BarInit()
if valtype(bPos) == "B"
   BARMOVEBLK := bPos
endif
return BARMOVEBLK


//==================================
// Determine if current tbrowse position is at 1st or last position
// and set internal object variable.  Returns relative position.
// This function works for non indexed databases and turns the
// vertical scrollbar display off, if an index is active.
// Use a custom function for indexed and non database tbrowses.
//
static function BarMoveDbf( obj)

obj:BarInit()
if indexord() != 0 .or. reccount() <= 1
   obj:BarStatus := SB_STATUS_INVALID
elseif recno() == 1
   obj:BarStatus := SB_STATUS_FIRST
elseif recno() == reccount()
   obj:BarStatus := SB_STATUS_LAST
else
   obj:BarStatus := SB_STATUS_VALID
endif
return (recno()-1)/(reccount()-1 )


//==================================
// Pans the display up without changing the browse cursor
// position, if possible.
//
function __SB_PanUp()
local obj   := qself()
local x, nSkip, nRows

DispBegin()
obj:BarInit()
obj:stabilize()
nRows := obj:rowpos-1
nSkip := abs(eval( obj:skipblock, -(nRows+1) ))
eval( obj:skipblock, nSkip )
if nSkip == nRows+1
   for x := 1 to nRows
      obj:Up()
   next
   obj:stabilize()
   obj:Up()
   obj:stabilize()
   for x := 1 to nRows
      obj:Down()
   next
   obj:stabilize()
   if obj:RowPos < obj:RowCount
      obj:Down()
      obj:stabilize()
   endif
endif
obj:stabilize()
DispEnd()
return qself()


//==================================
// Pans the display down without changing the browse cursor
// position, if possible.
//
function __SB_PanDown()
local obj   := qself()
local x, nSkip, nRows

DispBegin()
obj:BarInit()
obj:stabilize()
nRows := obj:rowcount() - obj:rowpos
nSkip := eval( obj:skipblock, nRows+1 )
eval( obj:skipblock, -nSkip )
if nSkip == nRows+1
   for x := 1 to nRows
      obj:down()
   next
   obj:stabilize()
   obj:down()
   obj:stabilize()
   for x := 1 to nRows
      obj:up()
   next
   obj:stabilize()
   if obj:RowPos > 1
      obj:up()
      obj:stabilize()
   endif
endif
obj:stabilize()
DispEnd()
return qself()


//==================================
