//Ŀ
//                                                                           
//  Program Name: ScrolBar.prg      Copyright: WardStar Consulting           
//  Date Created: 17/07/95           Language: Clipper 5.2e                  
//  Time Created: 18:41:10             Author: Robin Ward                    
//                                                                           
//   Description:   Adds scroll bars to any tbrowse operating on a database  
//                  file that is not indexed.                                
//                                                                           
//                  You can add your own code block to enable updating of    
//                  indexed or non database tbrowses.                        
//                                                                           
//                                                                           
//VerRevisedDescriptionĴ
//                                                                           
//   1.00  17/07/95    Created                                               
//                                                                           
//   1.01  30/07/95    Add PanUp and PanDown()                               
//                                                                           
//   1.02  28/08/95    Create a Sub Class of Tbrowse and alter the sub class 
//                     instead of altering the internal Tbrowse class        
//                                                                           
//
//   

#include "ScrolBar.ch"
#include "SubClass.ch"


// some defaults
#define  SB_FILL_CHAR         chr(177)
#define  SB_COLOR             "w/n"


// three object variables
#define  BARCOLOR             qself()[ len(qself())-2 ]
#define  BARSTATUS            qself()[ len(qself())-1 ]
#define  BARMOVEBLK           qself()[ len(qself())   ]


external __TSTABILI
external __TFORCEST


//==================================
function xTBrowseNew( nTop, nLeft, nBottom, nRight)
return xTBrowClass( nTop, nLeft, nBottom, nRight, 1)


//==================================
function xTBrowseDB( nTop, nLeft, nBottom, nRight)
return xTBrowClass( nTop, nLeft, nBottom, nRight, 2)


//==================================
// This function creates the extended tbrowse class,
// adds the new methods and variables, redfines two
// existing methods.
//
static function xTBrowClass( nTop, nLeft, nBottom, nRight, type)
static nHandle
local  nOldHandle
local  oXTbr

if nHandle == nil
   nHandle := SubClass( CLASS_TBROWSE, "XTBROWSE" )

   // add three public variables
   __ClassAdd( nHandle, "BarColor",    "__SB_Color"  )
   __ClassAdd( nHandle, "_BarColor",   "__SB_Color"  )
   __ClassAdd( nHandle, "BarMoveBlk",  "__SB_Move"   )
   __ClassAdd( nHandle, "_BarMoveBlk", "__SB_Move"   )
   __ClassAdd( nHandle, "BarStatus",   "__SB_Status" )
   __ClassAdd( nHandle, "_BarStatus",  "__SB_Status" )

   // add two methods
   __ClassAdd( nHandle, "BarUpdate",   "__SB_Update" )

   // add two more methods
   __ClassAdd( nHandle, "PanUp",       "__SB_PanUp"  )
   __ClassAdd( nHandle, "PanDown",     "__SB_PanDown")

   // add two more methods.  These are a duplicate of 
   // existing Stabilize and ForceStable methods
   __ClassAdd( nHandle, "xStabilize",   "__TSTABILI"  )
   __ClassAdd( nHandle, "xForceStable", "__TFORCEST"  )

   // redefine two existing methods
   __ClassAdd( nHandle, "Stabilize",    "__SB_xStab"  )
   __ClassAdd( nHandle, "ForceStable",  "__SB_xForce"  )

endif


// This code fools clipper into initialising our new objects
// as if they were standard clipper tbrowse objects

nOldHandle := __Class52H( CLASS_TBROWSE, nHandle )
if type == 1
   oXTbr := TBrowseNew( nTop, nLeft, nBottom, nRight)
else
   oXTbr := TBrowseDB( nTop, nLeft, nBottom, nRight)
endif
__Class52H( CLASS_TBROWSE, nOldHandle )


// Each tbrowse object has to initialized with the extra variables
// that are required for the scrolbar methods.  This extends the 
// length of the xtbrowse objects array by three variables.

aadd( oXTbr, SB_COLOR )
aadd( oXTbr, SB_STATUS_INVALID )
aadd( oXTbr, { |o| BarMoveDbf(o)} )
oXTbr:FootSep := SB_FILL_CHAR
if oXTbr:nRight == maxcol()
   oXTbr:nRight--
endif
oXTbr:configure()

return oXTbr


//==================================
// Updates the horizontal and vertical scrollbars.
// Call after the object has been stabilized.
//
function __SB_Update()
local obj      := qself()
local nTop     := obj:nBottom - obj:RowCount
local nBot     := obj:nBottom
local nLeft    := obj:nLeft
local nRight   := obj:nRight
local nHorCol
local nVertRow
local cPtr

DispBegin()

// draw the vertical scrolbar
@ nTop, nRight+1, nBot-1, nRight+1 box SB_FILL_CHAR

// calculate the vertical pointer position using the objects codeblock
nVertRow := (eval( obj:BarMoveBlk, obj) * (nBot-nTop-1 )) + 0.5

// is there a valid vertical pointer?
if obj:BarStatus != SB_STATUS_INVALID

   // choose the type of vertical pointer
   if obj:BarStatus == SB_STATUS_FIRST
      cPtr := chr(25)
   elseif obj:BarStatus == SB_STATUS_LAST
      cPtr := chr(24)
   else
      cPtr := chr(18)
   endif

   // display the vertical pointer
   @ nTop+nVertRow, nRight+1 say cPtr color obj:BarColor
endif


// draw the horizontal scrolbar
@ nBot, nLeft, nBot, nRight box SB_FILL_CHAR

// calculate the horizontal pointer position
nHorCol := (((obj:ColPos-1)/(obj:colcount-1)) * (nRight-nLeft)) + 0.5 + nLeft

// is there a valid horizontal pointer?
if obj:colcount > 1

   // choose the type of horizontal pointer
   if obj:ColPos == 1
      cPtr := chr(26)
   elseif obj:ColPos == obj:colcount
      cPtr := chr(27)
   else
      cPtr := chr(29)
   endif

   // display the horizontal pointer
   @ nBot, nHorCol say cPtr color obj:BarColor
endif

DispEnd()
return qself()


//==================================
function __SB_Color( cColor)

if valtype(cColor) == "C"
   BARCOLOR := cColor
endif
return BARCOLOR


//==================================
function __SB_Status( nStatus)

if valtype(nStatus) == "N"
   BARSTATUS := nStatus
endif
return BARSTATUS


//==================================
function __SB_Move( bPos)

if valtype(bPos) == "B"
   BARMOVEBLK := bPos
endif
return BARMOVEBLK


//==================================
// Determine if current tbrowse position is at 1st or last position
// and set internal object variable.  Returns relative position.
// This function works for non indexed databases and turns the
// vertical scrollbar display off, if an index is active.
// Use a custom function for indexed and non database tbrowses.
//
static function BarMoveDbf( obj)

if indexord() != 0 .or. reccount() <= 1
   obj:BarStatus := SB_STATUS_INVALID
elseif recno() == 1
   obj:BarStatus := SB_STATUS_FIRST
elseif recno() == reccount()
   obj:BarStatus := SB_STATUS_LAST
else
   obj:BarStatus := SB_STATUS_VALID
endif
return (recno()-1)/(reccount()-1 )


//==================================
// Pans the display up without changing the browse cursor
// position, if possible.
//
function __SB_PanUp()
local obj := qself()
local x, nSkip, nRows

DispBegin()
obj:xStabilize()
nRows := obj:rowpos-1
nSkip := abs(eval( obj:skipblock, -(nRows+1) ))
eval( obj:skipblock, nSkip )
if nSkip == nRows+1
   for x := 1 to nRows
      obj:Up()
   next
   obj:xStabilize()
   obj:Up()
   obj:xStabilize()
   for x := 1 to nRows
      obj:Down()
   next
   obj:xStabilize()
   if obj:RowPos < obj:RowCount
      obj:Down()
      obj:xStabilize()
   endif
endif
obj:xStabilize()
DispEnd()
return qself()


//==================================
// Pans the display down without changing the browse cursor
// position, if possible.
//
function __SB_PanDown()
local obj := qself()
local x, nSkip, nRows

DispBegin()
obj:xStabilize()
nRows := obj:rowcount() - obj:rowpos
nSkip := eval( obj:skipblock, nRows+1 )
eval( obj:skipblock, -nSkip )
if nSkip == nRows+1
   for x := 1 to nRows
      obj:down()
   next
   obj:xStabilize()
   obj:down()
   obj:xStabilize()
   for x := 1 to nRows
      obj:up()
   next
   obj:xStabilize()
   if obj:RowPos > 1
      obj:up()
      obj:xStabilize()
   endif
endif
obj:xStabilize()
DispEnd()
return qself()


//==================================
function __SB_xStab()
local obj := qself()

if obj:xStabilize()
   obj:BarUpdate()
   return .t.
endif
return .f.


//==================================
function __SB_xForce

return qself():xForceStable():BarUpdate()


//==================================

